#version 440 core

in float vf_water_height;
in vec4 vf_world_position;
in vec3 vf_cam_dir;

layout(location=0) out vec3 out_color;

uniform samplerCube env_map;

void main(void)
{
	// Compute Euclidean world position
	vec3 wp = vf_world_position.xyz / vf_world_position.w;
	// Compute world normal using screen-space derivatives
	vec3 wn = normalize(cross(normalize(dFdx(wp)), normalize(dFdy(wp))));
	vec3 ncd = normalize(vf_cam_dir);
	// Compute Fresnel reflection coefficient (Schlick's approximation)
	float costheta = dot(ncd,wn);
	float R0 = 0.6;
	float R = R0 + (1-R0)*pow(1+costheta,5);
	// Reflect view direction around normal
	vec3 refl = ncd - 2*costheta*wn;
	// Lookup environment map color
	out_color = texture(env_map, refl).rgb * R + (1-R)*vec3(0.0,0.0,0.2);
}