#version 440 core

layout(location=0) in vec2 position;
layout(location=1) in vec2 tex_coords;

uniform sampler2D tex;

out float vf_water_height;
out vec4 vf_world_position;
out vec3 vf_cam_dir;

uniform vec4 cam_position; // .xyz = camera positions, .w = camera sign
uniform vec3 cam_dir_ll;
uniform vec3 cam_dir_right;
uniform vec3 cam_dir_up;
uniform mat4 mvp_matrix;

// This shader draws a projected grid with node displacements according to wave height
void main(void)
{
	// Interpolate camera view direction in world space at current vertex
	vec3 cam_dir = cam_dir_ll + cam_dir_right * position.x + cam_dir_up * position.y;
	vf_cam_dir = cam_dir;

	// Find intersection between water plane (y=0) and camera view direction in projective coordinates
	vec4 pos = vec4(cam_dir.y * cam_position.xyz - cam_position.y * cam_dir, cam_dir.y) * cam_position.w;
	// Position of vertex in projective clip coordinates (without displacement) 
	vec4 glpos = mvp_matrix * pos;
	// Transform from clip coordinates to texture coordinates
	vec3 tex_coord = vec3(glpos.xy*0.5 + glpos.w*0.5, glpos.w);
	
	// Do projective texture lookup into wave height texture; division by homogeneous coordinate is taken care of by GPU
	vf_water_height = textureProj(tex, tex_coord).r;
	// Add displacement to vertex
	gl_Position = glpos + mvp_matrix[1] * vf_water_height * pos.w;

	vf_world_position = pos;
	vf_world_position[1] += vf_water_height * pos.w;
}