#pragma once

#include <memory>
#include <nanogui/opengl.h>

template<typename T>
class GLBuffer
{
public:
	template<int r, int c, int o, int mr, int mc>
	static auto makeMutable(GLenum target, const Eigen::Matrix<T, r, c, o, mr, mc>& data, GLenum usage)
	{
		auto buf = new GLBuffer<T>();
		buf->target = target;
		buf->usage = usage;
		buf->size = data.size() * sizeof(T);
		buf->num_elements = static_cast<unsigned int>(data.size());
		glGenBuffers(1, &buf->name);

		glBindBuffer(target, buf->name);
		glBufferData(target, buf->size, data.data(), usage);

		return std::unique_ptr<GLBuffer<T>>(buf);
	}

	static auto makeMutable(GLenum target, GLsizeiptr num_elements, GLenum usage)
	{
		auto buf = new GLBuffer<T>();
		buf->target = target;
		buf->usage = usage;
		buf->size = num_elements * sizeof(T);
		buf->num_elements = static_cast<unsigned int>(num_elements);
		glGenBuffers(1, &buf->name);

		glBindBuffer(target, buf->name);
		glBufferData(target, buf->size, nullptr, usage);

		return std::unique_ptr<GLBuffer<T>>(buf);
	}

	void bind()
	{
		glBindBuffer(target, name);
	}

	void unbind()
	{
		glBindBuffer(target, 0);
	}

	template<int r, int c, int o, int mr, int mc>
	void uploadData(const Eigen::Matrix<T, r, c, o, mr, mc>& data, bool autoshrink=false)
	{
		auto sz = data.size() * sizeof(T);
		glBindBuffer(target, name);
		if (autoshrink || static_cast<GLsizeiptr>(sz) > size)
		{
			glBufferData(target, sz, data.data(), usage);
			size = sz;
			num_elements = static_cast<unsigned int>(data.size());
		}
		else
		{
			glBufferSubData(target, 0, sz, data.data());
		}
	}

	~GLBuffer()
	{
		glDeleteBuffers(1, &this->name);
	}

	auto getName() const { return name; }
	auto getSize() const { return size; }
	auto getNumElements() const { return num_elements; }

	GLenum getGLType() const { return -1; }

private:
	GLBuffer() {}

	GLuint name;
	GLenum target;
	GLenum usage;
	GLsizeiptr size;
	unsigned int num_elements;
};

GLenum GLBuffer<float>::getGLType() const { return GL_FLOAT; }
GLenum GLBuffer<int>::getGLType() const { return GL_INT; }
GLenum GLBuffer<unsigned int>::getGLType() const { return GL_UNSIGNED_INT; }