#pragma once

#include <nanogui/opengl.h>
#include "GLShader.h"

using ShaderFromFile = std::pair<GLenum, std::string>;

class GLProgram
{
public:
	static auto makeFromShaders(std::initializer_list<std::shared_ptr<GLShader>> list)
	{
		auto prog = new GLProgram();

		for (auto elem : list)
		{
			prog->shaders.push_back(elem);
		}

		prog->name = glCreateProgram();

		return std::unique_ptr<GLProgram>(prog);
	}

	static auto makeFromFiles(std::initializer_list<std::pair<GLenum, std::string>> list)
	{
		auto prog = new GLProgram();

		for (auto elem : list)
		{
			prog->shaders.push_back(GLShader::makeFromFile(elem.first, elem.second));
		}

		prog->name = glCreateProgram();

		return std::unique_ptr<GLProgram>(prog);
	}

	bool link()
	{
		bool all_compiled = true;
		for (auto elem : shaders)
			all_compiled = all_compiled & elem->compile();
		if (!all_compiled)
			return false;

		for (auto elem : shaders)
			glAttachShader(name, elem->getName());

		glLinkProgram(name);
		GLint link_status;
		glGetProgramiv(name, GL_LINK_STATUS, &link_status);
		GLint log_length;
		glGetProgramiv(name, GL_INFO_LOG_LENGTH, &log_length);
		if (log_length > 1)
		{
			std::unique_ptr<char[]> infolog(new char[log_length]);
			glGetProgramInfoLog(name, log_length, nullptr, infolog.get());
			fprintf(stdout, "Program Info Log '%s': ", shaders[0]->getFileName().c_str());
			fprintf(stdout, link_status ? "Link succeeded!\n" : "Link failed!\n");
			fprintf(stdout, "%s\n", infolog.get());
		}
		linked = link_status;
		return linked;
	}

	void use() const
	{
		glUseProgram(name);
	}

	GLint getUniformLocation(const std::string& uni) const
	{
		return glGetUniformLocation(name, uni.c_str());
	}

	~GLProgram() { glDeleteProgram(name); }

private:
	bool linked{ false };
	GLuint name;
	std::vector<std::shared_ptr<GLShader>> shaders;
};