#pragma once

#include <memory>
#include <nanogui/opengl.h>
#include <fstream>
#include <sstream>

class GLShader
{
public:
	static auto makeFromFile(GLenum type, const std::string& filename)
	{
		std::ifstream fl;
		fl.open(filename);
		if (fl)
		{
			std::stringstream ss;
			ss << fl.rdbuf();

			auto shader = new GLShader();

			shader->filename = filename;
			shader->code = ss.str();
			shader->type = type;
			shader->name = glCreateShader(type);

			return std::shared_ptr<GLShader>(shader);
		}
		return std::shared_ptr<GLShader>(nullptr);
	}

	bool compile()
	{
		if (compiled)
			return true;

		auto codeptr = code.c_str();
		auto codelen = static_cast<GLint>(code.length());
		glShaderSource(name, 1, &codeptr, &codelen);
		glCompileShader(name);

		GLint status;
		glGetShaderiv(name, GL_COMPILE_STATUS, &status);
		GLint log_length;
		glGetShaderiv(name, GL_INFO_LOG_LENGTH, &log_length);
		if (log_length > 1)
		{
			std::unique_ptr<char[]> infolog(new char[log_length]);
			glGetShaderInfoLog(name, log_length, nullptr, infolog.get());
			fprintf(stdout, "Shader Info Log '%s': ", filename.c_str());
			fprintf(stdout, status ? "Compilation succeeded!\n" : "Compilation failed!\n");
			fprintf(stdout, "%s\n", infolog.get());
		}
		compiled = status;
		return compiled;
	}

	auto getName() { return name; }
	const auto& getFileName() { return filename; }

	~GLShader()
	{
		glDeleteShader(name);
	}

private:
	GLuint name;
	GLenum type;
	bool compiled{ false };
	std::string filename{ "" };
	std::string code;
};