#pragma once
#include <nanogui/nanogui.h>
#include <memory>

class GLTexture1D
{
private:
	GLTexture1D(GLuint name, GLint internalFormat, GLsizei width, bool immutable, GLenum format, GLenum type);

	GLsizei width;
	GLint minFilter, magFilter;
	GLint wrap;
	GLenum format, type;
	int num_channels;

public:
	~GLTexture1D();

	const GLuint name;
	const GLint internalFormat;

	inline GLsizei getWidth() const { return width; }
	inline GLint getMinFilter() const { return minFilter; }
	inline GLint getMagFilter() const { return magFilter; }
	inline GLint getWrap() const { return wrap; }

	static std::unique_ptr<GLTexture1D> newMutable(GLint internalFormat, GLsizei width, GLenum format, GLenum type, const void *data = nullptr);

	void setFilters(GLint minFilter, GLint magFilter);
	void setWrap(GLint wrap);
	void generateMipmaps() const;
	void bindAndActive(int index) const;
};

class GLTexture2D
{
private:
	GLTexture2D(GLuint name, GLint internalFormat, GLsizei width, GLsizei height, bool immutable, GLenum format, GLenum type);

	GLsizei width, height;
	GLint minFilter, magFilter;
	GLint wrap;
	GLenum format, type;
	int num_channels;

	template<typename T>
	void getImage(std::vector<T>& data, GLenum type, int& w, int& h, GLint level) const;

public:
	~GLTexture2D();

	const GLuint name;
	const GLint internalFormat;

	inline GLsizei getWidth() const { return width; }
	inline GLsizei getHeight() const { return height; }
	inline GLint getMinFilter() const { return minFilter; }
	inline GLint getMagFilter() const { return magFilter; }
	inline GLint getWrap() const { return wrap; }

	static std::unique_ptr<GLTexture2D> newMutable(GLint internalFormat, GLsizei width, GLsizei height, GLenum format, GLenum type, const void *data = nullptr);
	
	void setFilters(GLint minFilter, GLint magFilter);
	void setWrap(GLint wrap);
	void generateMipmaps() const;
	void bindAndActive(int index) const;
	void resize(GLsizei w, GLsizei h);

	void getImage(std::vector<unsigned char>& data, int& w, int& h, GLint level = 0) const;
	void getImage(std::vector<float>& data, int& w, int& h, GLint level = 0) const;
};

class GLTextureCubemap
{
private:
	GLTextureCubemap(GLuint name, GLint internalFormat, GLsizei width, GLenum format, GLenum type, bool immutable)
		: name(name), internalFormat(internalFormat), width(width), format(format), type(type), minFilter(GL_LINEAR), magFilter(GL_LINEAR) {}

	GLsizei width;
	GLint magFilter, minFilter;
	GLenum format, type;

public:
	~GLTextureCubemap();

	static std::unique_ptr<GLTextureCubemap> newMutable(GLint internalFormat, GLsizei width, GLenum format, GLenum type);

	void setContents(int side, void* data);

	void setFilters(GLint minFilter, GLint magFilter);
	void generateMipmaps() const;
	void bindAndActive(int index) const;

	inline GLsizei getWidth() const { return width; }
	inline GLint getMinFilter() const { return minFilter; }
	inline GLint getMagFilter() const { return magFilter; }

	const GLuint name;
	const GLint internalFormat;
};