#pragma once

#include <nanogui/opengl.h>
#include "GLBuffer.h"

class GLVertexArray
{
public:
	GLVertexArray()
	{
		glGenVertexArrays(1, &name);
	}

	template<typename T>
	void attachBuffer(const GLBuffer<T>& buffer, GLuint index, GLint size, GLuint divisor = 0)
	{
		glBindVertexArray(name);
		glBindBuffer(GL_ARRAY_BUFFER, buffer.getName());
		glEnableVertexAttribArray(index);
		glVertexAttribPointer(index, size, buffer.getGLType(), GL_FALSE, 0, nullptr);
		if (divisor > 0)
			glVertexAttribDivisor(index, divisor);
		glBindVertexArray(0);
	}

	void attachElementBuffer(const GLBuffer<unsigned int>& buffer)
	{
		glBindVertexArray(name);
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, buffer.getName());
		glBindVertexArray(0);
	}

	void bind() const
	{
		glBindVertexArray(name);
	}

	static void unbind()
	{
		glBindVertexArray(0);
	}

	~GLVertexArray()
	{
		glDeleteVertexArrays(1, &name);
	}

private:
	GLuint name;
};