#include "MeshHelper.h"

#include <iostream>
#include <unsupported\Eigen\KroneckerProduct>

void MeshHelper::makeRegularGrid(MeshOutput& output, int tx, int ty)
{
	auto xs = Eigen::Matrix<float, 1, -1>::LinSpaced(tx + 1, 0.f, 1.f);
	auto ys = Eigen::Matrix<float, 1, -1>::LinSpaced(ty + 1, 0.f, 1.f);

	output.positions = Eigen::Matrix<float, 2, -1>(2, (tx+1)*(ty+1));
	output.positions.row(0) << xs.replicate(1, ty + 1);
	auto temp = Eigen::Matrix<float, -1, -1>(ys.replicate(tx + 1, 1));
	output.positions.row(1) << Eigen::Matrix<float, 1, -1>(Eigen::Map<Eigen::Matrix<float, 1, -1>>(temp.data(), 1, (tx+1)*(ty+1)));

	auto ind = Eigen::Matrix<int, 3, -1>(3, 2 * tx*ty);
	Eigen::Matrix<int, 3, 2> it;
	it << 0, 0, 1, tx + 2, tx + 2, tx + 1;

	for (int y = 0; y < ty; y++)
	{
		auto seq = Eigen::Matrix<int, 1, -1>::LinSpaced(tx, (tx + 1)*y, (tx + 1)*y + tx - 1);
		auto temp = Eigen::kroneckerProduct(seq, Eigen::Matrix<int, 3, 2>::Ones());
		auto block = it.replicate(1, tx) + temp;
		ind.block(0, y*tx * 2, 3, 2 * tx) << block;
	}

	output.indices = ind.cast<unsigned int>();
}