#pragma once

#include <nanogui/nanogui.h>
#include <memory>
#include "Camera.h"
#include "GLdefines.h"
#include "WaterQuadRenderer.h"

class RippleApp : public nanogui::Screen
{
public:
	RippleApp(const nanogui::Vector2i& size);

	virtual void drawContents() override;
	static void printGLError();

	virtual bool mouseButtonEvent(const Eigen::Vector2i &p, int button, bool down, int modifiers) override;
	virtual bool mouseMotionEvent(const Eigen::Vector2i &p, const Eigen::Vector2i &rel, int button, int modifiers) override;
	virtual bool scrollEvent(const Eigen::Vector2i &p, const Eigen::Vector2f &rel) override;
	virtual bool keyboardEvent(int key, int scancode, int action, int modifier) override;
	virtual bool resizeEvent(const Eigen::Vector2i &s) override;
private:


	Eigen::Vector2f screen_size;

	ufBuffer grid_positions;
	uuBuffer grid_indices;
	uProgram basic_program;
	struct {
		GLint mvp_matrix;
		GLint cam_position;
		GLint cam_dir_ll;
		GLint cam_dir_right;
		GLint cam_dir_up;
		GLint tex;
		GLint env_map;
	} ul_basic_program;
	uVertexArray grid_vao;
	std::unique_ptr<Camera> cam;
	std::unique_ptr<Camera> cam2;
	bool camera_fixed = false;
	uTexture2D test_texture;
	uTexture2D water_height_texture;
	uTexture2D water_normal_texture;
	uFramebuffer water_height_framebuffer;
	uFramebuffer water_normal_framebuffer;
	uCubemap env_map;
	uProgram envmap_program;
	struct {
		GLint cam_dir_ll;
		GLint cam_dir_right;
		GLint cam_dir_up;
		GLint env_map;
	} ul_envmap_program;
	bool draw_waves{ true };
	bool update_waves{ true };
	bool rain_enabled{ false };
	int raindrops_per_second = 2;
	int raindrops_fallen = 0;
	double timestamp_rain_started = 0.0;


	ufBuffer quad_positions;
	uuBuffer quad_indices;
	uVertexArray quad_vao;

	nanogui::FormHelper* gui;
	nanogui::Window* gui_window;
	float drop_radius;
	bool capillary_waves, gravity_waves;
	bool left_pressed, ctrl_down;
	bool w_down{ false }, a_down{ false }, s_down{ false }, d_down{ false }, r_down{ false }, f_down{ false };
	bool wireframe = false;
	float samples_per_pixel_x, samples_per_pixel_y;
	int num_samples_x, num_samples_y;
	bool trigger_resize = false;
	double time_speed = 1.0;

	Eigen::Vector2f cam_angle_speed;
	float cam_scroll_speed;

	std::unique_ptr<WaterQuadRenderer> wqr;

	double last_time_stamp{ 0.0 };

	void addRipple(const Eigen::Vector2i &sp);
};