#pragma once

#include "Camera.h"
#include "GLdefines.h"
#include "WaveQuad.h"
#include "WaveComputations.h"

class WaterQuadRenderer
{
public:
	WaterQuadRenderer();

	void update(double dt);
	void drawWaves();
	void setCamera(Camera* cam) { this->cam = cam; }
	void addWave(const Eigen::Vector2f& pos);
	void setWaveTypes(bool const *gw, bool const *cw) {
		gravity_waves = gw; capillary_waves = cw;
	}

private:
	Camera * cam{ nullptr };

	uVertexArray quad_vao;
	ufBuffer quad_positions;
	uuBuffer quad_indices;
	ufBuffer quad_translations;
	bool const *gravity_waves{ nullptr }, *capillary_waves{ nullptr };

	uProgram wave_program;
	struct {
		GLint R_star;
		GLint mvp_matrix;
		GLint bound_ratio;
		GLint wave_height_texture;
	} ul_wave_program;

	std::list<WaveQuad> wave_quads;
	std::unique_ptr<WaveComputations> wc;

	double timestamp{ 0.0 };
	int wave_counter{ 0 };
};