#pragma once

#include <nanogui\nanogui.h>
#include "GLdefines.h"
#include "WaveQuad.h"

class WaveComputations
{
public:
	WaveComputations();

	void bindTextures(int k, int dV, int water_height) const;
	
	void computeWaveData(Eigen::Matrix<float, 3, -1>& data, const std::list<WaveQuad>& waves) const;
	void computeWaves(const std::list<WaveQuad>& waves, bool gw, bool cw);
	GLBuffer<float>* getWaveDataBuffer() { return per_wave_data_buffer.get(); }
	double getVMax() const { return V_max; }
	double getVMin() const { return V_min; }
	double getRStar() const { return R_star; }
	const GLTexture2D* getWaveHeightTexture() const { return wave_height_texture.get(); }

	const unsigned max_num_waves{ 1024u }, num_wave_samples{ 1024u };

private:
	double rho{ 1.0 }, g{ 980. }, R_star{ .2 }, It{ 273.61 }, tau_star{ 74. },
		nu{ 50e-3 };
	double tau, W, W_star, alpha, beta;
	double V_min, V_max;
	
	Eigen::VectorXf V_vals;
	Eigen::Matrix2Xf k_vals, dV_vals;
	
	uTexture1D k_texture, dV_texture;
	uTexture2D wave_height_texture;

	uProgram compute_wave_program;
	struct {
		GLint global_size;
		GLint V_bounds;

		GLint g;
		GLint R_star;
		GLint nu;
		GLint tau;
		GLint alpha;
		GLint wave_types;

		GLint k_texture;
		GLint dV_texture;
	} ul_compute_wave_program;
	ufBuffer per_wave_data_buffer;

	double sigma(double k);
	double V(double k);
	double dV(double k);
	double ddV(double k);
	void precomputeDominantWavenumbers();
};